% CRS4HCe - estimation of parameters in nonlinear regression
% Copyright (C) 2006  Josef Tvrdik
%
%This program is free software; you can redistribute it and/or
%modify it under the terms of the GNU General Public License
%as published by the Free Software Foundation; either version 2
%of the License, or (at your option) any later version.
%
%This program is distributed in the hope that it will be useful,
%but WITHOUT ANY WARRANTY; without even the implied warranty of
%MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%GNU General Public License for more details.
%
function [b_star, rss_star,  my_eps, ne, nrst]=...
    crs4hce(mod_name, y_obs, X, a, b,  my_eps0, gamma, N, max_evals, delta, w0)
% competing heuristics for non-linear regression,
% adaptive stopping condition
d=length(a);  h=4;  % number of heuristics in competition
if nargin ~= 11
    N=10*d; delta=1/(h*5); w0=0.5;  max_evals=40000; % default setting
end
if nargin==5
    my_eps0=1e-9; gamma=1e7;
end
% initialization
P=zeros(N,d+1);
for i=1:N
    P(i,1:d)=a+(b-a).*rand(1,d);
    P(i,d+1)= rss(mod_name,P(i,1:d),y_obs,X);
end  % 0-th generation initialized
[fmax, indmax]=max(P(:,d+1));
[fmin, indmin]=min(P(:,d+1));
ne=N;  nrst=0; wi=zeros(1,h)+w0;
tss=(y_obs-mean(y_obs))'*(y_obs-mean(y_obs));
my_eps=my_eps0;
tss_myeps=tss*my_eps;
one_minus_R2=0;
prosel=0; % indicate passing inner while loop
while one_minus_R2 < my_eps*gamma && prosel==0 && gamma>=10
    while (fmax-fmin > tss_myeps) && (ne < d*max_evals) %inner main loop
        hh=roulette_simple(wi);
        switch hh % number of selected heuristic
            case 1
                y=refl_rwd(P,[2 0.5]);
            case 2
                y=refl_rwd(P,[5 1.5]);
            case 3
                y=refl_bestrwd(P, 2, 0.5, indmin, P(indmin,1:d));
            case 4
                y=decrs_radp(P,[0.4 0.9 fmin fmax]);
        end
        y=zrcad(y,a,b);     % perturbation
        fy=rss(mod_name,y,y_obs,X);
        ne=ne+1;
        if fy < fmax  % trial point y is good for renewing population
            P(indmax,:)=  [y fy];
            if fmax-fmin <= eps    w=w0;
            else w=(fmax-max([fy fmin]))/(fmax-fmin); % weight of increase, <= 1
            end
            if isnan(w) w=w0; end
            wi(hh)=wi(hh)+w; % change of qi
            p_min=min(wi)/sum(wi);
            if p_min<delta
                wi=zeros(1,h)+w0;
                nrst=nrst+1;
            end  %reset
            [fmax, indmax]=max(P(:,d+1));
            [fmin, indmin]=min(P(:,d+1));
        end
        prosel=1;
    end   % inner main loop - end
    if prosel==0 gamma=gamma/10; end
    one_minus_R2=fmin/tss;
    %    fmax-fmin
    if one_minus_R2<my_eps*gamma && prosel==1
        my_eps=my_eps*0.1;
        tss_myeps=tss*my_eps;
        prosel=0;
    end
end % outer main loop - end
b_star=P(indmin,1:d);
rss_star=fmin;
if prosel==0
    my_eps=my_eps*10;
end
%